pub struct SliceParallelSource<'data, T> { /* private fields */ }Expand description
A parallel source over a slice. This struct is created by the
par_iter() method on
IntoExactParallelRefSource.
You most likely won’t need to interact with this struct directly, as it
implements the ExactParallelSource and
ExactParallelSourceExt traits, but it is
nonetheless public because of the must_use annotation.
See also MutSliceParallelSource.
let input = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
let iter: SliceParallelSource<_> = input.par_iter();
let sum = iter.with_thread_pool(&mut thread_pool).sum::<i32>();
assert_eq!(sum, 5 * 11);The slice element type must be Sync, so the following example doesn’t
compile.
ⓘ
use std::cell::Cell;
let input = [Cell::new(1), Cell::new(2), Cell::new(3), Cell::new(4)];
let iter: SliceParallelSource<_> = input.par_iter();Trait Implementations
Sourceimpl<'data, T: Sync> ExactParallelSource for SliceParallelSource<'data, T>
impl<'data, T: Sync> ExactParallelSource for SliceParallelSource<'data, T>