ThreadPool

Struct ThreadPool 

pub struct ThreadPool { /* private fields */ }
Available on (crate features rayon or default-thread-pool) and crate feature default-thread-pool only.
A thread pool that can execute parallel pipelines.

This type doesn’t expose any public methods other than num_threads(). You can interact with it via the ThreadPoolBuilder::build() function to create a thread pool, and the with_thread_pool() method to attach a thread pool to a parallel iterator.

Implementations

Source

impl ThreadPool

pub fn num_threads(&self) -> NonZeroUsize

Returns the number of worker threads that have been spawned in this thread pool.